@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2006 Jonh Wendell <wendell@bani.com.br> 
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +19,7 @@
  *
  * Authors:
  *      Mark McLoughlin <mark@skynet.ie>
+ *      Jonh Wendell <wendell@bani.com.br>
  */
 
 #include <config.h>
@@ -29,11 +31,13 @@
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-icon-theme.h>
-#include <libgnomevfs/gnome-vfs-init.h>
-#include "vino-url.h"
+#include <libgnome/libgnome.h>
+#include <libgnomeui/libgnomeui.h>
+#include <dbus/dbus-glib.h>
 
-#define _(x) dgettext (GETTEXT_PACKAGE, x)
+#ifdef VINO_ENABLE_KEYRING
+#include <gnome-keyring.h>
+#endif
 
 #define VINO_PREFS_DIR                    "/desktop/gnome/remote_access"
 #define VINO_PREFS_ENABLED                VINO_PREFS_DIR "/enabled"
@@ -45,6 +49,9 @@
 
 #define N_LISTENERS 6
 
+#define VINO_DBUS_BUS_NAME  "org.gnome.Vino"
+#define VINO_DBUS_INTERFACE "org.gnome.VinoScreen"
+
 typedef struct {
   GladeXML    *xml;
   GConfClient *client;
@@ -62,181 +69,80 @@ typedef struct {
   GtkWidget   *password_toggle;
   GtkWidget   *password_box;
   GtkWidget   *password_entry;
+  GtkTooltips *tips;
 
   guint        listeners [N_LISTENERS];
   int          n_listeners;
+  int          expected_listeners;
 
   guint        use_password : 1;
 } VinoPreferencesDialog;
 
+static void
+vino_preferences_dialog_update_url_label (VinoPreferencesDialog *dialog);
+
 static char *
-vino_preferences_dialog_base64_encode (const char *data)
+vino_preferences_dialog_get_password_from_keyring (VinoPreferencesDialog *dialog)
 {
-#define CHARS_PER_LINE 72
+#ifdef VINO_ENABLE_KEYRING
+  GnomeKeyringNetworkPasswordData *found_item;
+  GnomeKeyringResult               result;
+  GList                           *matches;
+  char                            *password;
+  
+  matches = NULL;
 
-  static const char *to_base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-  char              *retval, *p;
-  int                length;
-  int                div, rem;
-  int                chars, newlines;
+  result = gnome_keyring_find_network_password_sync (
+                NULL,           /* user     */
+		NULL,           /* domain   */
+		"vino.local",   /* server   */
+		NULL,           /* object   */
+		"rfb",          /* protocol */
+		"vnc-password", /* authtype */
+		5900,           /* port     */
+		&matches);
 
-  if (!data || (length = strlen (data)) <= 0)
+  if (result != GNOME_KEYRING_RESULT_OK)
     return NULL;
 
-  div      = length / 3;
-  rem      = length % 3;
-  chars    = (div * 4) + rem + 2;
-  newlines = (chars + CHARS_PER_LINE - 1) / CHARS_PER_LINE;
+  g_assert (matches != NULL && matches->data != NULL);
 
-  retval = p = g_new0 (char, chars + newlines + 1);
+  found_item = (GnomeKeyringNetworkPasswordData *) matches->data;
 
-  chars = 0;
-  while (div-- > 0)
-    {
-      p [0] = to_base64 [ (data [0] >> 2) & 0x3f];
-      p [1] = to_base64 [((data [0] << 4) & 0x30) + ((data [1] >> 4) & 0xf)];
-      p [2] = to_base64 [((data [1] << 2) & 0x3c) + ((data [2] >> 6) & 0x3)];
-      p [3] = to_base64 [  data [2] & 0x3f];
+  password = g_strdup (found_item->password);
 
-      data += 3;
-      p    += 4;
+  gnome_keyring_network_password_list_free (matches);
 
-      if ((chars += 4) == CHARS_PER_LINE)
-	{
-	  chars = 0;
-	  *(p++) = '\n';
-	}
-    }
-
-  switch (rem)
-    {
-    case 2:
-      p [0] = to_base64 [ (data [0] >> 2) & 0x3f];
-      p [1] = to_base64 [((data [0] << 4) & 0x30) + ((data [1] >> 4) & 0xf)];
-      p [2] = to_base64 [ (data [1] << 2) & 0x3c];
-      p [3] = '=';
-      break;
-    case 1:
-      p [0] = to_base64 [(data [0] >> 2) & 0x3f];
-      p [1] = to_base64 [(data [0] << 4) & 0x30];
-      p [2] = '=';
-      p [3] = '=';
-      break;
-    }
-
-  return retval;
-
-#undef CHARS_PER_LINE
+  return password;
+#else
+  return NULL;
+#endif
 }
 
-static char *
-vino_preferences_dialog_base64_unencode (const char *data)
+static gboolean
+vino_preferences_dialog_set_password_in_keyring (VinoPreferencesDialog *dialog,
+                                                 const char            *password)
 {
-  static const char *to_base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-  const        char *p;
-  char              *retval;
-  int                i, length, count, rem;
-  int                qw, tw;
+#ifdef VINO_ENABLE_KEYRING
+  GnomeKeyringResult result;
+  guint32            item_id;
 
-  if (!data || (length = strlen (data)) <= 0)
-    return NULL;
+  result = gnome_keyring_set_network_password_sync (
+                NULL,           /* default keyring */
+                NULL,           /* user            */
+                NULL,           /* domain          */
+                "vino.local",   /* server          */
+                NULL,           /* object          */
+                "rfb",          /* protocol        */
+                "vnc-password", /* authtype        */
+                5900,           /* port            */
+                password,       /* password        */
+                &item_id);
 
-  p = data;
-  count = rem = 0;
-  while (length > 0)
-    {
-      int skip, vrfy, i;
-
-      skip = strspn (p, to_base64);
-
-      count  += skip;
-      p      += skip;
-      length -= skip;
-
-      if (length <= 0)
-	break;
-
-      vrfy = strcspn (p, to_base64);
-      
-      for (i = 0; i < vrfy; i++)
-	{
-	  if (g_ascii_isspace (p [i]))
-	    continue;
-
-	  if (p [i] != '=')
-	    return NULL;
-	  
-	  /* rem must be either 2 or 3, otherwise
-	   * no '=' should be here
-	   */
-	  if ((rem = count % 4) < 2)
-	    return NULL;
-
-	  /* end-of-message */
-	  break;
-	}
-
-      length -= vrfy;
-      p      += vrfy;
-    }
-
-  retval = g_new0 (char, (count / 4) * 3 + (rem ? rem - 1 : 0) + 1);
-
-  if (count <= 0)
-    return retval;
-
-  qw = tw = 0;
-  for (i = 0; data [i]; i++)
-    {
-      char c = data [i];
-      char bits;
-
-      if (g_ascii_isspace (c))
-	continue;
-
-      bits = 0;
-      if ((c >= 'A') && (c <= 'Z'))
-	{
-	  bits = c - 'A';
-	}
-      else if ((c >= 'a') && (c <= 'z'))
-	{
-	  bits = c - 'a' + 26;
-	}
-      else if ((c >= '0') && (c <= '9'))
-	{
-	  bits = c - '0' + 52;
-	}
-      else if (c == '=')
-	{
-	  break;
-	}
-
-      switch (qw++)
-	{
-	case 0:
-	  retval [tw + 0] = (bits << 2) & 0xfc;
-	  break;
-	case 1:
-	  retval [tw + 0] |= (bits >> 4) & 0x03;
-	  retval [tw + 1]  = (bits << 4) & 0xf0;
-	  break;
-	case 2:
-	  retval [tw + 1] |= (bits >> 2) & 0x0f;
-	  retval [tw + 2]  = (bits << 6) & 0xc0;
-	  break;
-	case 3:
-	  retval [tw + 2] |= bits & 0x3f;
-	  qw = 0;
-	  tw +=3;
-	  break;
-	default:
-	  g_assert_not_reached ();
-	  break;
-	}
-    }
-
-  return retval;
+  return result == GNOME_KEYRING_RESULT_OK;
+#else
+  return FALSE;
+#endif
 }
 
 static void
@@ -548,17 +454,30 @@ vino_preferences_vnc_password_notify (GConfClient           *client,
 				      VinoPreferencesDialog *dialog)
 {
   const char *password_b64;
+  guchar     *blob;
+  gsize       blob_len;
   char       *password;
 
   if (!entry->value || entry->value->type != GCONF_VALUE_STRING)
     return;
 
+  password = NULL;
+
   password_b64 = gconf_value_get_string (entry->value);
-  password = vino_preferences_dialog_base64_unencode (password_b64);
+
+  if (password_b64)
+    {
+      blob_len = 0;
+      blob = g_base64_decode (password_b64, &blob_len);
+    
+      password = g_strndup ((char *) blob, blob_len);
+
+      g_free (blob);
+    }
 
   if (!password || !password [0])
     {
-      gtk_entry_set_text (GTK_ENTRY (dialog->password_entry), NULL);
+      gtk_entry_set_text (GTK_ENTRY (dialog->password_entry), "");
     }
   else
     {
@@ -582,6 +501,10 @@ vino_preferences_dialog_password_changed (GtkEntry              *entry,
   const char *password;
 
   password = gtk_entry_get_text (entry);
+
+  if (vino_preferences_dialog_set_password_in_keyring (dialog, password))
+    return;
+
   if (!password || !password [0])
     {
       gconf_client_unset (dialog->client, VINO_PREFS_VNC_PASSWORD, NULL);
@@ -590,7 +513,7 @@ vino_preferences_dialog_password_changed (GtkEntry              *entry,
     {
       char *password_b64;
 
-      password_b64 = vino_preferences_dialog_base64_encode (password);
+      password_b64 = g_base64_encode ((guchar *) password, strlen (password));
 
       gconf_client_set_string (dialog->client, VINO_PREFS_VNC_PASSWORD, password_b64, NULL);
 
@@ -601,8 +524,8 @@ vino_preferences_dialog_password_changed (GtkEntry              *entry,
 static void
 vino_preferences_dialog_setup_password_entry (VinoPreferencesDialog *dialog)
 {
-  char *password_b64;
-  char *password;
+  char     *password;
+  gboolean  password_in_keyring;
 
   dialog->password_entry = glade_xml_get_widget (dialog->xml, "password_entry");
   g_assert (dialog->password_entry != NULL);
@@ -610,15 +533,35 @@ vino_preferences_dialog_setup_password_entry (VinoPreferencesDialog *dialog)
   dialog->password_box = glade_xml_get_widget (dialog->xml, "password_box");
   g_assert (dialog->password_box != NULL);
 
-  password_b64 = gconf_client_get_string (dialog->client, VINO_PREFS_VNC_PASSWORD, NULL);
-  password = vino_preferences_dialog_base64_unencode (password_b64);
+  password_in_keyring = TRUE;
+
+  if (!(password = vino_preferences_dialog_get_password_from_keyring (dialog)))
+    {
+      guchar *blob;
+      gsize   blob_len;
+      char   *password_b64;
+
+      password_b64 = gconf_client_get_string (dialog->client, VINO_PREFS_VNC_PASSWORD, NULL);
+
+      if (password_b64)
+        {
+           blob_len = 0;
+           blob = g_base64_decode (password_b64, &blob_len);
+
+           password = g_strndup ((char *) blob, blob_len);
+        
+           g_free (blob);
+           g_free (password_b64);
+        }
+
+      password_in_keyring = FALSE;
+    }
 
   if (password)
     {
       gtk_entry_set_text (GTK_ENTRY (dialog->password_entry), password);
     }
 
-  g_free (password_b64);
   g_free (password);
 
   g_signal_connect (dialog->password_entry, "changed",
@@ -626,78 +569,170 @@ vino_preferences_dialog_setup_password_entry (VinoPreferencesDialog *dialog)
 
   gtk_widget_set_sensitive (dialog->password_box, dialog->use_password);
 
-  if (!gconf_client_key_is_writable (dialog->client, VINO_PREFS_VNC_PASSWORD, NULL))
+  if (!password_in_keyring)
     {
-      gtk_widget_set_sensitive (dialog->password_box, FALSE);
-      gtk_widget_show (dialog->writability_warning);
-    }
+      if (!gconf_client_key_is_writable (dialog->client, VINO_PREFS_VNC_PASSWORD, NULL))
+        {
+          gtk_widget_set_sensitive (dialog->password_box, FALSE);
+          gtk_widget_show (dialog->writability_warning);
+        }
 
-  dialog->listeners [dialog->n_listeners] = 
-    gconf_client_notify_add (dialog->client,
-			     VINO_PREFS_VNC_PASSWORD,
-			     (GConfClientNotifyFunc) vino_preferences_vnc_password_notify,
-			     dialog, NULL, NULL);
-  dialog->n_listeners++;
+      dialog->listeners [dialog->n_listeners] = 
+        gconf_client_notify_add (dialog->client,
+                                 VINO_PREFS_VNC_PASSWORD,
+                                 (GConfClientNotifyFunc) vino_preferences_vnc_password_notify,
+                                 dialog, NULL, NULL);
+      dialog->n_listeners++;
+    }
+  else
+    {
+      dialog->expected_listeners--;
+    }
 }
 
 static void
 vino_preferences_dialog_setup_icons (VinoPreferencesDialog *dialog)
 {
-#define ICON_SIZE_STANDARD 48
-
-  GnomeIconTheme *icon_theme;
-  char           *icon_path;
+  gtk_window_set_icon_name (GTK_WINDOW (dialog->dialog), "gnome-remote-desktop");
 
   dialog->sharing_icon = glade_xml_get_widget (dialog->xml, "sharing_icon");
   g_assert (dialog->sharing_icon != NULL);
   
+  gtk_image_set_from_icon_name (GTK_IMAGE (dialog->sharing_icon),
+                                "gnome-remote-desktop",
+                                GTK_ICON_SIZE_DIALOG);
+
   dialog->security_icon = glade_xml_get_widget (dialog->xml, "security_icon");
   g_assert (dialog->security_icon != NULL);
 
-  icon_theme = gnome_icon_theme_new ();
-  gnome_icon_theme_set_allow_svg (icon_theme, TRUE);
+  gtk_image_set_from_icon_name (GTK_IMAGE (dialog->security_icon),
+                                "gnome-lockscreen",
+                                GTK_ICON_SIZE_DIALOG);
+}
 
-  icon_path = gnome_icon_theme_lookup_icon (icon_theme,
-					    "gnome-remote-desktop",
-					    ICON_SIZE_STANDARD,
-					    NULL, NULL);
-  if (icon_path)
-    {
-      gtk_window_set_icon_from_file (GTK_WINDOW (dialog->dialog), icon_path, NULL);
-      gtk_image_set_from_file (GTK_IMAGE (dialog->sharing_icon), icon_path);
-      g_free (icon_path);
-    }
+static void
+vino_preferences_server_updated (DBusGProxy *proxy,
+                                 const char *name,
+                                 const char *prev_owner,
+                                 const char *new_owner,
+                                 gpointer user_data)
+{
+  if ( (new_owner) && (!strcmp (name, VINO_DBUS_BUS_NAME)) )
+    vino_preferences_dialog_update_url_label ( (VinoPreferencesDialog *) user_data);
+}
 
-  icon_path = gnome_icon_theme_lookup_icon (icon_theme,
-					    "gnome-lockscreen",
-					    ICON_SIZE_STANDARD,
-					    NULL, NULL);
-  if (icon_path)
-    {
-      gtk_image_set_from_file (GTK_IMAGE (dialog->security_icon), icon_path);
-      g_free (icon_path);
-    }
-
-  g_object_unref (icon_theme);
+static void
+vino_preferences_start_listening (VinoPreferencesDialog *dialog)
+{
+  DBusGConnection *connection;
+  GError          *error;
+  DBusGProxy      *proxy;
   
-#undef ICON_SIZE_STANDARD
+  error = NULL;
+  connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+  if (!connection)
+    {
+      g_printerr (_("Failed to open connection to bus: %s\n"),
+                  error->message);
+      g_error_free (error);
+      return;
+    }
+
+  proxy = dbus_g_proxy_new_for_name (connection,
+                                     DBUS_SERVICE_DBUS,
+                                     DBUS_PATH_DBUS,
+                                     DBUS_INTERFACE_DBUS);
+
+  dbus_g_proxy_add_signal (proxy, "NameOwnerChanged",
+                           G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
+  dbus_g_proxy_connect_signal (proxy, "NameOwnerChanged",
+                          G_CALLBACK (vino_preferences_server_updated), dialog, NULL);
+}
+
+static int
+vino_preferences_get_server_port (VinoPreferencesDialog *dialog)
+{
+#define VINO_DEFAULT_PORT   5900
+#define VINO_MIN_PORT       5000
+#define VINO_MAX_PORT       6000
+
+  DBusGConnection *connection;
+  GError          *error;
+  DBusGProxy      *proxy;
+  int              port;
+  GdkScreen       *screen;
+  char            *obj_path;
+  
+  error = NULL;
+  connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+  if (!connection)
+    {
+      g_printerr (_("Failed to open connection to bus: %s\n"),
+                  error->message);
+      g_error_free (error);
+      return 0;
+    }
+
+  screen = gtk_window_get_screen (GTK_WINDOW (dialog->dialog));
+
+  obj_path = g_strdup_printf ("/org/gnome/vino/screens/%d",
+                              gdk_screen_get_number (screen));
+
+  proxy = dbus_g_proxy_new_for_name (connection,
+                                     VINO_DBUS_BUS_NAME,
+                                     obj_path,
+                                     VINO_DBUS_INTERFACE);
+
+  g_free (obj_path);
+
+  if (!dbus_g_proxy_call (proxy, "GetServerPort", &error,
+                          G_TYPE_INVALID,
+                          G_TYPE_INT, &port,
+                          G_TYPE_INVALID))
+    {
+      g_object_unref (proxy);
+      g_error_free (error);
+      dbus_g_connection_unref (connection);
+      return 0;
+    }
+
+  g_object_unref (proxy);
+  dbus_g_connection_unref (connection);
+
+  if (port >= VINO_MIN_PORT && port <= VINO_MAX_PORT)
+    return port - VINO_DEFAULT_PORT;
+  else
+    return port;
+
+#undef VINO_DEFAULT_PORT
+#undef VINO_MIN_PORT
+#undef VINO_MAX_PORT
 }
 
 static char *
 vino_preferences_get_local_hostname (void)
 {
-  static char local_host [NI_MAXHOST] = { 0, };
+  static char      local_host [NI_MAXHOST] = { 0, };
+  struct addrinfo  hints;
+  struct addrinfo *results;
+  char            *retval;
 
-  if (gethostname (local_host, NI_MAXHOST) != -1)
-    {
-      struct hostent *host;
-      
-      host = gethostbyname (local_host);
+  if (gethostname (local_host, NI_MAXHOST) == -1)
+    return NULL;
 
-      return g_strdup (host ? host->h_name : local_host);
-    }
+  memset (&hints, 0, sizeof (hints));
+  hints.ai_flags = AI_CANONNAME;
 
-  return NULL;
+  results = NULL;
+  if (getaddrinfo (local_host,  NULL, &hints, &results) != 0)
+    return NULL;
+
+  retval = g_strdup (results ? results->ai_canonname : local_host);
+
+  if (results)
+    freeaddrinfo (results);
+
+  return retval;
 }
 
 static char *
@@ -705,15 +740,17 @@ vino_preferences_dialog_get_server_command (VinoPreferencesDialog *dialog)
 {
   char *local_host;
   char *server_url;
+  int   server_port;
+
+  local_host  = vino_preferences_get_local_hostname ();
+  server_port = vino_preferences_get_server_port (dialog);
 
-  local_host = vino_preferences_get_local_hostname ();
   if (!local_host)
     {
-      return g_strdup ("vncviewer localhost:0");
+      return g_strdup_printf ("vncviewer localhost:%d", server_port);
     }
 
-  /* FIXME: get the actual port number for the server on this screen */
-  server_url = g_strdup_printf ("vncviewer %s:0", local_host);
+  server_url = g_strdup_printf ("vncviewer %s:%d", local_host, server_port);
 
   g_free (local_host);
 
@@ -745,8 +782,8 @@ vino_preferences_dialog_update_url_label (VinoPreferencesDialog *dialog)
   command = vino_preferences_dialog_get_server_command (dialog);
   mailto = vino_preferences_dialog_construct_mailto (dialog, command);
 
-  gtk_label_set_text (GTK_LABEL (dialog->url_label), command);
-  vino_url_set_address (VINO_URL (dialog->url_label), mailto);
+  gtk_button_set_label (GTK_BUTTON (dialog->url_label), command);
+  gtk_link_button_set_uri (GTK_LINK_BUTTON (dialog->url_label), mailto);
   
   g_free (command);
   g_free (mailto);
@@ -779,6 +816,35 @@ vino_preferences_dialog_mailto_notify (GConfClient           *client,
     }
 }
 
+static void
+vino_preferences_dialog_uri_hook (GtkLinkButton *button,
+                                  const gchar *link,
+                                  gpointer user_data)
+{
+  GError *error;
+  GdkScreen *screen;
+
+  screen = gtk_widget_get_screen (GTK_WIDGET (button));
+
+  error = NULL;
+  if (!gnome_url_show_on_screen (link, screen, &error))
+    {
+      /* FIXME better error handling!
+       *       What best to do? For the specific case
+       *       in this preferences dialog we want to be
+       *       able to pop up a dialog with the error
+       *       but also the vino URL as a selectable
+       *       label.
+       *
+       *       Maybe chain this up to the caller?
+       */
+
+      g_warning ("Failed to show URL '%s': %s\n",
+		 link, error->message);
+      g_error_free (error);
+    }
+}
+
 static void
 vino_preferences_dialog_setup_url_labels (VinoPreferencesDialog *dialog)
 {
@@ -805,13 +871,18 @@ vino_preferences_dialog_setup_url_labels (VinoPreferencesDialog *dialog)
   command = vino_preferences_dialog_get_server_command (dialog);
   mailto = vino_preferences_dialog_construct_mailto (dialog, command);
   
-  dialog->url_label = vino_url_new (mailto, command,
-				    _("Send this command by email"));
+  gtk_link_button_set_uri_hook (vino_preferences_dialog_uri_hook, NULL, NULL);
+
+  dialog->url_label = gtk_link_button_new_with_label (mailto, command);
+
+  dialog->tips = gtk_tooltips_new ();
+  gtk_tooltips_set_tip (dialog->tips, dialog->url_label,
+                        _("Send this command by email"), NULL);
+  g_object_ref_sink (dialog->tips);
+
   g_free (command);
   g_free (mailto);
 
-  gtk_misc_set_alignment (GTK_MISC (dialog->url_label), 0.0, 0.0);
-
   gtk_box_pack_start (GTK_BOX (dialog->url_labels_box),
 		      dialog->url_label,
 		      FALSE, FALSE, 0);
@@ -823,15 +894,35 @@ vino_preferences_dialog_response (GtkWidget             *widget,
 				  int                    response,
 				  VinoPreferencesDialog *dialog)
 {
-  switch (response)
+  GError *error;
+
+  if (response != GTK_RESPONSE_HELP)
     {
-    case GTK_RESPONSE_CLOSE:
-    default:
       gtk_widget_destroy (widget);
-      break;
-    case GTK_RESPONSE_HELP:
-      /* FIXME: hook up the help docs when they're written */
-      break;
+      return;
+    }
+
+  error = NULL;
+  gnome_help_display_desktop (NULL, "user-guide", "user-guide.xml", "goscustdesk-90", &error);
+  if (error)
+    {
+      GtkWidget *message_dialog;
+
+      message_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog->dialog),
+					       GTK_DIALOG_DESTROY_WITH_PARENT,
+					       GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_CLOSE,
+					       _("There was an error displaying help:\n%s"),
+					       error->message);
+      gtk_window_set_resizable (GTK_WINDOW (message_dialog), FALSE);
+
+      g_signal_connect (message_dialog, "response",
+			G_CALLBACK (gtk_widget_destroy),
+			NULL);
+
+      gtk_widget_show (message_dialog);
+
+      g_error_free (error);
     }
 }
 
@@ -852,6 +943,8 @@ vino_preferences_dialog_init (VinoPreferencesDialog *dialog)
   const char *glade_file;
   gboolean    allowed;
 
+  dialog->expected_listeners = N_LISTENERS;
+
   if (g_file_test (VINO_GLADE_FILE, G_FILE_TEST_EXISTS))
     glade_file = VINO_GLADE_FILE;
   else
@@ -891,7 +984,7 @@ vino_preferences_dialog_init (VinoPreferencesDialog *dialog)
   vino_preferences_dialog_setup_password_toggle       (dialog);
   vino_preferences_dialog_setup_password_entry        (dialog);
 
-  g_assert (dialog->n_listeners == N_LISTENERS);
+  g_assert (dialog->n_listeners == dialog->expected_listeners);
 
   vino_preferences_dialog_update_for_allowed (dialog, allowed);
 
@@ -913,6 +1006,10 @@ vino_preferences_dialog_finalize (VinoPreferencesDialog *dialog)
     g_free (dialog->mailto);
   dialog->mailto = NULL;
 
+  if (dialog->tips)
+    g_object_unref (dialog->tips);
+  dialog->tips = NULL;
+
   if (dialog->client)
     {
       int i;
@@ -941,24 +1038,23 @@ main (int argc, char **argv)
 {
   VinoPreferencesDialog dialog = { NULL, };
 
-  gtk_init (&argc, &argv);
-  gnome_vfs_init ();
-
   bindtextdomain (GETTEXT_PACKAGE, VINO_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
+		      argc, argv, NULL);
+
   if (!vino_preferences_dialog_init (&dialog))
     {
       vino_preferences_dialog_finalize (&dialog);
       return 1;
     }
 
+  vino_preferences_start_listening (&dialog);
   gtk_main ();
 
   vino_preferences_dialog_finalize (&dialog);
-  
-  gnome_vfs_shutdown ();
 
   return 0;
 }
